import { Http, I18n } from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';
import { GenericValueController } from '@/controllers/GenericValueController';
import ChangeLabel from '@/helper/Labels';

export class GraphicsController {
  /**
 * Populate a pie chart component with data and update its options.
 *
 * This function is responsible for populating a pie chart component with data and updating its options.
 * The data is fetched and then is processed to extract labels and series values for the chart.
 * Finally, the component's series and chart options are updated based on the retrieved information.
 *
 * @param {Object} component - The specific pie chart component to be populated and updated.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public fillGraphicPie({ component }) {
    const props = component.options.props;
    const routeName = props.routeName;
    const seriesNameColumn = props.seriesNameColumn;
    const labelsNameColumn = props.labelsNameColumn;
    const formatting = new Utilities();
    const newLabels = [];
    const newSeries = [];
    const sendObject = {
      filter: [],
    };
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (props.idOrigem) {
      sendObject.filter.push(
        {
          name: 'IDORIGEM',
          operator: '=',
          value: props.idOrigem,
        },
      );
    }
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        /* eslint no-console: ["error", { allow: ["warn", "error", "info", "log"] }] */
        const data = response.data.dataset[routeName];
        data.forEach((row) => {
          newLabels.push(ChangeLabel.getLabel(row[labelsNameColumn]));
          newSeries.push(row[seriesNameColumn]);
        });
        component.series = newSeries;
        const options = {
          labels: newLabels,
          yaxis: {
            labels: {
              formatter: (value) => (
                formatting.formatValue('float', value)
              ),
            },
          },
        };
        component.updateChart(options);
      });
    } else {
      const genericValues = new GenericValueController();
      const data = genericValues.valueGraphicPie[component.name].slice(0, 10);
      
      data.forEach((row) => {
        newLabels.push(ChangeLabel.getLabel(row[labelsNameColumn]));
        newSeries.push(row[seriesNameColumn]);
      });
      component.series = newSeries;
      const options = {
        labels: newLabels,
        yaxis: {
          labels: {
            formatter: (value) => (
              formatting.formatValue('float', value)
            ),
          },
        },
      };
      component.updateChart(options);
    }
  }
  
  /**
 * Populate a bar chart component with data and update its options.
 *
 * This function is responsible for populating a bar chart component with data and updating its options.
 * The data is fetched and then is processed to extract x-axis labels and series values for the chart.
 * Finally, the component's series and chart options are updated based on the retrieved information.
 * In the end of the function, if I have a graphic bar on the screen, it will immediatly change the color
 * of every draggable-hander component.
 *
 * @param {Object} component - The specific bar chart component to be populated and updated.
 *
 * @access public
 * @authot Jefferson Gouvea
 */
  public async fillGraphicBar({ component }) {
    const props = component.options.props;
    const routeName = props.routeName;
    const xaxisNameColumn = props.xaxisNameColumn;
    const seriesProps = props.series;
    const formatting = new Utilities();
    const changeColorDragComponent = new Utilities();
    const limit = props.limit ? props.limit : 10;
    const sendObject = {
      filter: [],
    };
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (genericTest === 'false' || genericTest === null) {
      if (props.idOrigem) {
        sendObject.filter.push(
          {
            name: 'IDORIGEM',
            operator: '=',
            value: props.idOrigem,
          },
        );
      } else if (props.complete) {
        sendObject.filter.push(
          {
            name: 'COMPLETE',
            operator: '=',
            value: props.complete,
          },
        );
      }
      
      Http.post(`/${routeName}`, sendObject).then((response) => {
        const data = response.data.dataset[routeName].slice(0, limit);
        const xaxis = [];
        const newSeries = [];
        Object.keys(seriesProps).forEach((prop) => {
          newSeries.push({
            name: I18n.translate(seriesProps[prop].label),
            data: data.map((item) => item[prop]),
          });
        });
        
        data.forEach((row) => { xaxis.push(row[xaxisNameColumn].substring(0, 15)); });
        component.series = newSeries;
        const options = {
          yaxis: {
            labels: {
              formatter: (value) => (
                formatting.formatValue('float', value)
              ),
            },
          },
          xaxis: {
            categories: xaxis,
          },
        };
        component.updateChart(options);
      });
    } else {
      const data = genericValues.valueGraphicBar[component.name].slice(0, limit);
      const xaxis = [];
      const newSeries = [];
      Object.keys(seriesProps).forEach((prop) => {
        newSeries.push({
          name: seriesProps[prop].label,
          data: data.map((item) => item[prop]),
        });
      });

      data.forEach((row) => { xaxis.push(row[xaxisNameColumn].substring(0, 15)); });
      component.series = newSeries;
      const options = {
        yaxis: {
          labels: {
            formatter: (value) => (
              formatting.formatValue('float', value)
            ),
          },
        },
        xaxis: {
          categories: xaxis,
        },
      };
      component.updateChart(options);
    }
    
    const zdDragHandler = document.querySelectorAll('.draggable-handle');
    let pathElement: any = 0;
    pathElement = zdDragHandler[0].querySelector('svg path');
    if (pathElement.getAttribute('fill')) {
      changeColorDragComponent.changeDragHandlerColor();
    }
  }
  
  /**
 * Populate a gauge chart component with data and update its options.
 *
 * This function is responsible for populating a gauge chart component with data and updating its options.
 * The data is fetched and then is processed to extract information for the gauge chart,
 * including title, subtext, and color configuration for the axis line. Finally, the component's chart
 * options are updated based on the retrieved information.
 *
 * @param {Object} component - The specific gauge chart component to be populated and updated.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public fillGraphicGauge({ component }) {
    const props = component.options.props;
    const routeName = props.routeName;
    const sendObject = {
      filter: [],
    };
    
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        const data = response.data.dataset[routeName][0];
        const options = {
          title: {
            text: `${data.DSINDICADOR} - ${data.DATAAPURADO}`,
            subtext: `(Meta: ${data.VRLIMMETA}%)`,
          },
          series: [
            {
              axisLine: {
                lineStyle: {
                  color: [
                    [data.VRLIMMINIMO / 100, '#F44336'],
                    [data.VRLIMMETA / 100, '#F1C232'],
                    [1, '#6AA84F'],
                  ],
                },
              },
              data: [
                {
                  value: data.ENTREGA_TOTAL,
                },
              ],
            },
          ],
        };
        component.updateChart(options);
      });
    } else {
      const data = genericValues.valueGraphicGauge[routeName][0];
      
      const options = {
        series: [
          {
            axisLine: {
              lineStyle: {
                color: [
                  [data.VRLIMMINIMO / 100, '#F44336'],
                  [data.VRLIMMETA / 100, '#F1C232'],
                  [1, '#6AA84F'],
                ],
              },
            },
            data: [
              {
                value: data.ENTREGA_TOTAL,
              },
            ],
          },
        ],
      };
      component.updateChart(options);
    }
  }
  
  /**
 * Populate a data table component with formatted data and update it.
 *
 * This function is responsible for populating a data table component with formatted data and updating it.
 * The data is fetched and then is processed to format numerical values and update the table's data
 * source(grid modal for bar chart).
 *
 * @param {Object} component - The specific data table component to be populated and updated.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public tableData({ component }) {
    const formatting = new Utilities();
    const routeName = component.name;
    const sendObject = {
      filter: [],
    };
    let data;
    
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        data = response.data.dataset[routeName].map((obj) => {
          Object.keys(obj).forEach((key) => {
            if (typeof obj[key] === 'number') {
              obj[key] = formatting.formatValue('float', obj[key]);
            }
          });
        
          return obj;
        });
        component.datasource.updateData(data);
      });
    } else {
      data = genericValues.valueGraphicBar[routeName].map((obj) => {
        Object.keys(obj).forEach((key) => {
          if (typeof obj[key] === 'number') {
            obj[key] = formatting.formatValue('float', obj[key]);
          }
        });
      
        return obj;
      });
      
      component.datasource.updateData(data);
    }
  }

  /**
 * Populate a data table component with selected data and update it.
 *
 * This function is responsible for populating a data table component with selected data and updating it.
 * The selected data is filtered and fetched and then is processed to format numerical values and update
 * the table's data source(grid modal for pie chart).
 *
 * @param {Object} component - The specific data table component to be populated and updated with selected data.
 *
 * @access public
 * @authot Jefferson Gouvea
 */
  public tableForSelectedData({ component }) {
    const formatting = new Utilities();
    const routeName = component.name;
    const selectedData = component.datasource.filter.SELECTED;
    const sendObject = {
      filter: [
        {
          name: 'SELECTED',
          operator: '=',
          value: selectedData,
        },
      ],
    };
    let data;
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        data = response.data.dataset[routeName].map((obj) => {
          Object.keys(obj).forEach((key) => {
            if (typeof obj[key] === 'number') {
              obj[key] = formatting.formatValue('float', obj[key]);
            }
          });
        
          return obj;
        });
        component.datasource.setFilter([]);
        component.datasource.updateData(data);
      });
    } else {
      const genericValues = new GenericValueController();
      data = genericValues.valueGrid[component.name].map((obj) => {
        Object.keys(obj).forEach((key) => {
          if (typeof obj[key] === 'number') {
            obj[key] = formatting.formatValue('float', obj[key]);
          }
        });
      
        return obj;
      });
      
      component.datasource.updateData(data);
    }
  }
}
